# -*- coding: utf-8 -*-

# passerelle - uniform access to multiple data sources and services
# Copyright (C) 2019  Entr'ouvert
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from django.db import models
from django.utils.six.moves import urllib
from django.utils.translation import ugettext_lazy as _
import lxml.etree
from zeep import helpers

from passerelle.base.models import BaseResource
from passerelle.utils.api import endpoint
from passerelle.utils.jsonresponse import APIError


INSERT_DEMANDE_BY_TYPE_SCHEMA = {
    "$schema": "http://json-schema.org/draft-03/schema#",
    "title": "",
    "description": "",
    "type": "object",
    "properties": {
        "contact_nom": {
            "description": "Nom du contact",
            "required": True
        },
        "contact_tel": {
            "description": "Téléphone du contact",
            "type": "string",
        },
        "contact_email": {
            "description": "Email du contact",
            "type": "string",
        },
        "contact_adresse": {
            "description": "Adresse du contact",
            "type": "string",
        },
        "demande_objet": {
            "description": "Objet de la demande",
            "type": "string",
        },
        "demande_lieu": {
            "description": "Lieu de la demande",
            "type": "string",
        },
        "demande_description": {
            "description": "Description de la demande",
            "type": "string",
        },
        "remote_adresse": {
            "description": "",
            "type": "string"
        },
        "code_equipement": {
            "description": "Code de l'équipement",
            "type": "string"
        },
        "code_service_demandeur": {
            "description": "Code du service demandeur",
            "type": "string"
        },
        "date_souhaite": {
            "description": "Date souhaitée",
            "type": "string"
        },
        "type_demande": {
            "description": "Type demande",
            "type": "string"
        }
    }
}


class ATALConnector(BaseResource):
    base_soap_url = models.URLField(
        max_length=400, verbose_name=_('Base SOAP endpoint'),
        help_text=_('URL of the base SOAP endpoint'))
    category = _('Business Process Connectors')

    class Meta:
        verbose_name = _('ATAL connector')

    def _soap_call(self, wsdl, method, **kwargs):
        wsdl_url = urllib.parse.urljoin(self.base_soap_url, '%s?wsdl' % wsdl)
        client = self.soap_client(wsdl_url=wsdl_url)
        return getattr(client.service, method)(**kwargs)

    def _basic_ref(self, wsdl, method):
        soap_res = self._soap_call(wsdl=wsdl, method=method)
        res = []
        for elem in soap_res:
            res.append({'id': elem.code, 'text': elem.libelle})
        return {'data': res}

    @endpoint(methods=['get'], perm='can_access')
    def get_type_activite(self, request):
        return self._basic_ref('VilleAgileService', 'getTypeActivite')

    @endpoint(methods=['get'], perm='can_access')
    def get_type_de_voie(self, request):
        return self._basic_ref('VilleAgileService', 'getTypeDeVoie')

    @endpoint(methods=['get'], perm='can_access')
    def get_types_equipement(self, request):
        soap_res = self._soap_call(wsdl='VilleAgileService', method='getTypesEquipement')
        tree = lxml.etree.fromstring(soap_res.encode('utf-8')).getroottree()
        types = tree.xpath('//types')[0]
        res = []
        for type_elem in types.getchildren():
            res.append({'id': type_elem.get('id'), 'text': type_elem.get('label')})
        return {'data': res}

    @endpoint(
        perm='can_access',
        post={
            'description': _('Insert Demande By Type'),
            'request_body': {
                'schema': {
                    'application/json': INSERT_DEMANDE_BY_TYPE_SCHEMA
                }
            }
        }
    )
    def insert_demande_by_type(self, request, post_data):
        demande_number = self._soap_call(
            wsdl='DemandeService', method='insertDemandeByType',
            contactNom=post_data['contact_nom'],
            contactTelephone=post_data['contact_telephone'],
            contactCourriel=post_data['contact_email'],
            contactAdresse=post_data['contact_adresse'], demandeObjet=post_data['demande_objet'],
            demandeLieu=post_data['demande_lieu'],
            demandeDescription=post_data['demande_description'],
            remoteAddress=post_data['remote_adresse'], codeEquipement=post_data['code_equipement'],
            codeServiceDemandeur=post_data['code_service_demandeur'],
            dateSouhaitee=post_data['date_souhaite'], typeDemande=post_data['type_demande']
        )
        return {'data': {'demande_number': demande_number}}

    @endpoint(
        methods=['get'], perm='can_access', example_pattern='{demande_number}/',
        pattern='^(?P<demande_number>\w+)/$',
        parameters={
            'demande_number': {
                'description': _('Demande number'), 'example_value': 'DIT18050001'
            }
        }
    )
    def retrieve_details_demande(self, request, demande_number, **kwargs):
        if not demande_number:
            raise APIError('A demande_number parameter must be specified')
        soap_res = self._soap_call(
            wsdl='DemandeService', method='retrieveDetailsDemande',
            demandeNumberParam=demande_number)
        return {'data': helpers.serialize_object(soap_res)}
