from django.contrib.contenttypes.models import ContentType
import mock
import pytest

from passerelle.apps.atal.models import ATALConnector
from passerelle.base.models import ApiUser, AccessRight


@pytest.fixture()
def connector(db):
    api = ApiUser.objects.create(username='all', keytype='', key='')
    connector = ATALConnector.objects.create(
        base_soap_url='http://example.atal.com/', slug='slug-atal')
    obj_type = ContentType.objects.get_for_model(connector)
    AccessRight.objects.create(
        codename='can_access', apiuser=api, resource_type=obj_type, resource_pk=connector.pk)
    return connector


def mock_atal_soap_call(monkeypatch, return_value=None, side_effect=None):
    kwargs = {}
    if return_value is not None:
        kwargs['return_value'] = return_value
    if side_effect is not None:
        kwargs['side_effect'] = side_effect
    mock_soap_call = mock.Mock(**kwargs)
    monkeypatch.setattr(ATALConnector, '_soap_call', mock_soap_call)
    return mock_soap_call


class SoapElem(object):

    def __init__(self, **kwargs):
        for attr, value in kwargs.items():
            setattr(self, attr, value)


REFS = [
    SoapElem(code='code1', libelle='elem1'), SoapElem(code='code2', libelle='elem2')
]


def test_get_type_activite(app, connector, monkeypatch):
    mock_soap_call = mock_atal_soap_call(monkeypatch, return_value=REFS)
    response = app.get('/atal/slug-atal/get_type_activite')
    assert response.json == {
        'err': 0,
        'data': [
            {'text': 'elem1', 'id': 'code1'},
            {'text': 'elem2', 'id': 'code2'}]
    }
    call_params = mock_soap_call.call_args.kwargs
    assert call_params['wsdl'] == 'VilleAgileService'
    assert call_params['method'] == 'getTypeActivite'


def test_get_type_de_voie(app, connector, monkeypatch):
    mock_soap_call = mock_atal_soap_call(monkeypatch, return_value=REFS)
    response = app.get('/atal/slug-atal/get_type_de_voie')
    assert response.json == {
        'err': 0,
        'data': [
            {'text': 'elem1', 'id': 'code1'},
            {'text': 'elem2', 'id': 'code2'}]
    }
    call_params = mock_soap_call.call_args.kwargs
    assert call_params['wsdl'] == 'VilleAgileService'
    assert call_params['method'] == 'getTypeDeVoie'


def test_get_types_equipement(app, connector, monkeypatch):
    return_value = u"""<?xml version="1.0" encoding="UTF-8" standalone="no"?>
    <types>
        <type id="2" label="Espaces Verts"></type>
        <type id="4" label="Voirie">
    </type>
</types>
"""
    mock_soap_call = mock_atal_soap_call(monkeypatch, return_value=return_value)
    response = app.get('/atal/slug-atal/get_types_equipement')
    assert response.json == {
        'err': 0,
        'data': [
            {'text': 'Espaces Verts', 'id': '2'},
            {'text': 'Voirie', 'id': '4'}]
    }
    call_params = mock_soap_call.call_args.kwargs
    assert call_params['wsdl'] == 'VilleAgileService'
    assert call_params['method'] == 'getTypesEquipement'


def test_insert_demande_by_type(app, connector, monkeypatch):
    mock_soap_call = mock_atal_soap_call(monkeypatch, return_value='DIT19050001')
    params = {
        'contact_nom': 'John Doe',
        'contact_telephone': '0101010101',
        'contact_email': 'john@doe.com',
        'contact_adresse': '1 doe street',
        'demande_objet': 'sarah connor',
        'demande_lieu': 'LA',
        'demande_description': 'poker face',
        'remote_adresse': 'hollywood bd',
        'code_equipement': 'MAC10',
        'code_service_demandeur': 'skynet',
        'date_souhaite': 'now',
        'type_demande': 'scary'
    }
    response = app.post_json('/atal/slug-atal/insert_demande_by_type', params=params)
    assert response.json == {
        'err': 0,
        'data': {'demande_number': 'DIT19050001'}
    }
    call_params = mock_soap_call.call_args.kwargs
    assert call_params['wsdl'] == 'DemandeService'
    assert call_params['method'] == 'insertDemandeByType'
    assert call_params['contactNom'] == 'John Doe'
    assert call_params['contactTelephone'] == '0101010101'
    assert call_params['contactCourriel'] == 'john@doe.com'
    assert call_params['contactAdresse'] == '1 doe street'
    assert call_params['demandeObjet'] == 'sarah connor'
    assert call_params['demandeLieu'] == 'LA'
    assert call_params['demandeDescription'] == 'poker face'
    assert call_params['remoteAddress'] == 'hollywood bd'
    assert call_params['codeEquipement'] == 'MAC10'
    assert call_params['codeServiceDemandeur'] == 'skynet'
    assert call_params['dateSouhaitee'] == 'now'
    assert call_params['typeDemande'] == 'scary'


def test_retrieve_details_demande(app, connector, monkeypatch):
    mock_soap_call = mock_atal_soap_call(
        monkeypatch, return_value=dict(code='code1', libelle='elem1'))
    response = app.get('/atal/slug-atal/retrieve_details_demande/DIT19050001/')
    assert response.json == {
        'err': 0,
        'data': {'code': 'code1', 'libelle': 'elem1'}
    }
    call_params = mock_soap_call.call_args.kwargs
    assert call_params['wsdl'] == 'DemandeService'
    assert call_params['method'] == 'retrieveDetailsDemande'
    assert call_params['demandeNumberParam'] == 'DIT19050001'
